/*
NAME: ThunderBrowse Global
AUTH: GameSpotting! Networks
DESC: Contains all the global ThunderBrowse functions and varibles
*/

const Ccl = Components.classes;
const Cil = Components.interfaces;
var tbrowser;
var ThunderTabBrowser;

var thunderbrowseprefs = Ccl["@mozilla.org/preferences-service;1"].getService(Cil.nsIPrefService).getBranch("extensions.tbrowse.");

var TBrowseGlobal = {
  
  beta: false,
  app: null,
  isSpice: false,
  isPostBox: false,
  isPostExpress: false,
  isThunder: false,
  appTabsAllowed: false,
  tbrowseVersion: 0,
  mainWin: null,
 /* Starting Protocol lists */
  tbitcmds: (/(^tbit:changelog|^tbit:commands|^tbit:donate|^tbit:about|^tbit:faq|^tbit:options|^tbit:license|^dl|^addons|^home|^options)/i),
  aboutcmds: (/(^about:config|^about:plugins|^about:kitchensink|^about:config|^about:blank|^about:downloads|^about:mozilla|^about:credits|^about:rights|^about:crashes|^about:cache|^about:commands|^about:license|^about:buildconfig|^about:addons)/i),
  mailProtocols: (/(^mailbox|^pop3|^pop|^imap|^news|^snews|^addbook|^mailto)/i),
 /* A seprate check is done for about and tbit URLs so they are not listed here */
  otherProtocols: (/(^javascript|^custombutton|^file|^resource|^chrome|^source|^view-source|^whois|^tbit|^about|^http(s)?)/i),
  obServ: Ccl["@mozilla.org/observer-service;1"].getService(Cil.nsIObserverService),

 checkBrowser : function()
 {
	tbrowser = null;
	var windowType = null;
	//define the right browser
	if(document.documentElement.hasAttribute("windowtype") == true)
		windowType = document.documentElement.getAttribute("windowtype");
	else
	{
		TBrowseGlobal.tbLogMsg("Error! Tried to bind to unsupported or null window.");
		return false;
	}
	if(windowType == "mail:3pane" || windowType == "mail:messageWindow" || windowType == "collab:main" || windowType == "tbrowse:win")
	{
		try
		{
			var brows = (ThunderTabBrowser) ? ThunderTabBrowser.getBrowserForTab(ThunderTabBrowser.selectedTab) : null;
			if(brows)
			{
				tbrowser = brows;
				return tbrowser;
			}
		}
		catch(e)
		{
			TBrowseGlobal.tbLogMsg("Could not find browser, returning default. Error: " + e);
			tbrowser = ($("ThunderBrowsePopup")) ? $("ext-messagepane") : $("messagepane");
			return tbrowser;	
		}
	}
  },
  getTabType: function()
  {
   var tabtype = Components.classes["@mozilla.org/preferences-service;1"]
       		.getService(Components.interfaces.nsIPrefService)
		       .getBranch("extensions.tbrowse.").getIntPref("tabtype");
   var winType = document.documentElement.getAttribute("windowtype");
   if(winType == "mail:messageWindow" || winType == "tbrowse:win")
    return 0;
   else
    return tabtype;  
  },
  viewSource: function (url)
  {
     var realURL = (TBrowseGlobal.contextHasLink()) ? TBrowseGlobal.getContextLink() : url;
	      window.openDialog( "chrome://global/content/viewSource.xul",
                               "_blank",
                               "scrollbars,resizable,chrome,dialog=no",
                               realURL,
                               "UTF-8");
		return true;
  },
  newWindow : function(newUrl)
  {
	if(!newUrl)
		var url = TBrowseGlobal.getCurrentURL();
	else
		var url = newUrl;
	var mailProtoChck = (url.search(TBrowseGlobal.mailProtocols) == 0);	
	if(url != null && url != "" && url != "options"  && (/(^tbit|^about)/i).test(url) == false && mailProtoChck == false)
	{
		window.openDialog("chrome://tbrowse/content/tbwin.xul",
                   "_blank","scrollbars,resizable,status,chrome,dialog=no",url);
	}

	else
		window.openDialog("chrome://tbrowse/content/tbwin.xul",
		            "_blank","status,scrollbars,resizable,chrome,dialog=no"); 
  },
  getTabHandler: function()
  {
	  //Basically, this returns the id for using TB3 tabs
		return (this.isSpice == true) ? document.getElementById("tabcollab") : document.getElementById("tabmail");  
  },
  getVersion: function()
  {
    if(this.tbrowseVersion != 0)
      return this.tbrowseVersion;

    try
    {
        Components.utils.import("resource://gre/modules/AddonManager.jsm");
        AddonManager.getAddonByID("ThunderBrowse@thunderbrowse.com", function(addon) {
        this.tbrowseVersion = addon.version;
        });
    }
    catch(ex)
    {
        var gExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
                            .getService(Components.interfaces.nsIExtensionManager);
          this.tbrowseVersion = gExtensionManager.getItemForID("ThunderBrowse@thunderbrowse.com").version;
    }

    return this.tbrowseVersion;
  },
  updateTabsCheck: function()
  {
	if((this.isThunder == true && TBrowseDev.underTB3() == false) || (this.isSpice == true && TBrowseDev.compareAppVersions("0.8") == true))
		this.appTabsAllowed = true;
	else if(this.isPostbox == true && this.isPostExpress == false && TBrowseDev.compareAppVersions("2.0") == true)
		this.appTabsAllowed = false;
	else
		this.appTabsAllowed = false;  
  },
  checkApplication: function()
  {
	 var spicebirdId = "{ee53ece0-255c-4cc6-8a7e-81a8b6e5ba2c}";
	 var thunderbirdId = "{3550f703-e582-4d05-9a08-453d09bdfdc6}";
	 var postboxId = "postbox@postbox-inc.com";
	 var expressBoxId = "express@postbox-inc.com";
	 var appId = Components.classes["@mozilla.org/xre/app-info;1"]
	     	   .getService(Components.interfaces.nsIXULAppInfo).ID;
	 if(appId == thunderbirdId)
	 {
	   this.app = "thunderbird";
		this.isThunder = true;
	   this.mainWin = "mail:3pane";  
	 }
	 if(appId == postboxId || appId == expressBoxId)
	 {
	   this.app = "postbox";
	   this.mainWin = "mail:3pane";
	   this.isPostBox = true;
		this.isPostExpress = (appId == expressBoxId);
	 }
	 if(appId == spicebirdId)
	 {
	   this.app = "spicebird";
	   this.isSpice = true;
	   this.mainWin = "collab:main";
	 }
  },
  getCurrentURL: function()
  {
       /*
        The difference between this function and the one in TBrowseDev 
	is that this one also checks the url bar for the current url.
	While the other one just grabs it from the current document
	displayed.
       */ 
	checkBrowser();
	if(tbrowser)
	{
		if(gTBURLBox.value != null && gTBURLBox.value != " ")
			return gTBURLBox.value;
		else
			return tbrowser.currentURI.spec;
	}
	else
	  return gTBURLBox.value;
  },
  getCharset : function(fromBrowser)
  {
     //Because of how annoying and helpful all the internal functions are, we will always get utf-8.
     return (fromBrowser.ownerDocument.characterSet) ? fromBrowser.ownerDocument.characterSet : "UTF-8"; 
  },
  openWindowFromContent: function()
  {
		TBrowseGlobal.newWindow(TBrowseGlobal.getContextLink());
  },
  whatsNew: function()
  {
     var pref =
      Components.classes["@mozilla.org/preferences-service;1"].
        getService(Components.interfaces.nsIPrefBranch);
   if(pref)
   {
     var version = this.getVersion();
     var curRegistered = pref.getCharPref("extensions.tbrowse.install");
     var firstime = pref.getBoolPref("extensions.tbrowse.firstimerun");
     var shownew = pref.getBoolPref("extensions.tbrowse.shownew");
     if(curRegistered != version && firstime == false && shownew == true) 
   {
    TBrowseGlobal.newWindow("http://new.thunderbrowse.com/");
      pref.setCharPref("extensions.tbrowse.install", version);
     }
   }
  },
  contextHasLink: function()
  {
    if(gContextMenu && gContextMenu.onLink)
      return true;
    else if(document.popupNode)
    {
      if(document.popupNode.hasAttribute("href"))
        return true;
      else if(document.popupNode.parentNode && document.popupNode.parentNode.hasAttribute("href"))
        return true;
      
      return false;
    }  
    else
      return false;
  
  },
  getContextLink: function()
  {
    if(gContextMenu && gContextMenu.onLink)
      return ('getLinkURL' in gContextMenu) ? gContextMenu.getLinkURL() : gContextMenu.linkURL();
    else if(document.popupNode)
    {
      if(document.popupNode.hasAttribute("href"))
        return document.popupNode.href;
      else if(document.popupNode.parentNode && document.popupNode.parentNode.hasAttribute("href"))
        return document.popupNode.parentNode.href;
    }  
    else
      return TBrowseGlobal.getCurrentURL();
  },
  checkSourceLink: function()
  {
	try{
 	 if(TBrowseGlobal.contextHasLink())
    	 $('urlbarsource-messagePaneContext').setAttribute("disabled",true);
 	else
  	 $('urlbarsource-messagePaneContext').setAttribute("disabled",false); 
	 }
 	catch(e){  $('urlbarsource-messagePaneContext').setAttribute("disabled",true);}
  },
  checkContextLink: function(event)
  {
   var shouldHide = TBrowseGlobal.contextHasLink();
	 var menuItems = ["urlbar-messagePaneContext","urlbarwin-messagePaneContext",
	     	       "urlbartab-messagePaneContext"];
	 for(var i = 0; i < menuItems.length; i++)
	  $(menuItems[i]).setAttribute("hidden",!shouldHide);
   
  },
  tbLogMsg: function(msg)
  {
     var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
     consoleService.logStringMessage("ThunderBrowse: " + msg);  
  }
};
/* AutoSuggest */
var TBAutoSuggest = {
     resultClick: function()
     {
	var url = gTBURLBox.getResultValueAt(gTBURLBox.resultsPopup.selectedIndex);
	if(thunderbrowseprefs.getBoolPref("ACfillBar") == false)
		ThunderBrowse.loadURL(url); 
	else
		gTBURLBox.setTextValue(url);	
	gTBURLBox.closeResultPopup();
     },
     watchKeyboardCodes : function(e)
     {
	if(e.keyCode == KeyEvent.DOM_VK_RETURN || e.keyCode == KeyEvent.DOM_VK_ENTER 
		     || e.keyCode == KeyEvent.DOM_VK_DOWN)
	{
		if(e.keyCode == KeyEvent.DOM_VK_RETURN || e.keyCode == KeyEvent.DOM_VK_ENTER)
			ThunderBrowse.loadURL();
		//This reveals a history menu
		if(e.keyCode == KeyEvent.DOM_VK_DOWN && ((gTBURLBox.mMenuOpen == false 
  || gTBURLBox.mMenuOpen == null) && (gTBURLBox.resultsPopup.mMenuOpen == false 
  || gTBURLBox.resultsPopup.mMenuOpen == null)))
			TBHistory.generateDropdown();
		return e; 		
	}
     }
};
function $(obj) 
{
    return document.getElementById(obj);
}
//rename this
var listObserver = {
  onDragStart: function (event, transferData, action) {
    var txt = event.target.getAttribute("href");
    transferData.data = new TransferData();
    transferData.data.addDataForFlavour("text/html", txt);
  }
};
//Override function
function openURL(that)
{
TBrowseGlobal.checkApplication();
var winQ = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
var gettbrowserwindow = winQ.getMostRecentWindow(TBrowseGlobal.mainWin);
var canLoadLinks = TBrowsePermissions.linksAllowed();
if(canLoadLinks == true && gettbrowserwindow)
{	
	if(thunderbrowseprefs.getBoolPref("linkWind") == true)
	 gettbrowserwindow.openDialog("chrome://tbrowse/content/tbwin.xul","_blank",
			"scrollbars,resizable,status,chrome,dialog=no",that);
	else
	 gettbrowserwindow.ThunderTabBrowser.addTab(that);
}
else if(gettbrowserwindow)
	gettbrowserwindow.ThunderTabBrowser.extBrows(that); 
}
