/*
NAME: ThunderBrowse Data Capture
AUTH: GameSpotting! Networks
DESC: This file deals with capturing and loading links and dealing with special
      content. Data submission and link handling occurs here.
*/
var postbrowserfortab;
var formbaseURL;

var ThunderBrowseHandler = {
   addOrig: true,
   nodeId: null,
   gestureHold: false,
	keySubmit: false,
   lastCrawl: null,
   lastCrawlResult: null,
   /* This crawls the DOM tree to find elements we can use */
   treeCrawler: function(event)
   {
    if(event.target == this.lastCrawl)
      return this.lastCrawlResult;
    else
    {  
    var parent = null, nodeClk = null;
    var target = event.target;
    if(target instanceof HTMLAnchorElement ||
	    target instanceof HTMLAreaElement ||
	    target instanceof HTMLLinkElement ||
	    target instanceof HTMLInputElement ||
	    target instanceof HTMLButtonElement ||
	    target instanceof HTMLOptionElement ||
          target.type == "submit" ||
          target.type == "image"  ||
          target.type == "button" ||
	    target.form) 
      {
	 if(target.hasAttribute("href") || 
            target.hasAttribute("onclick") ||
	    target.parentNode.hasAttribute("onchange") || 
	    target.form)
 	     nodeClk = target;
      }
	if(nodeClk == null)
	{    
	 parent = target.parentNode;
	 while (parent && nodeClk == null) 
	 {
	  if ((parent instanceof HTMLAnchorElement ||
	      parent instanceof HTMLAreaElement ||
	      parent instanceof HTMLLinkElement ||	    
	      parent instanceof HTMLInputElement ||
	      parent instanceof HTMLButtonElement ||
            parent.type == "submit" ||
            parent.type == "image"  ||
            parent.type == "button" ||
	      parent.form) && nodeClk == null)
	   {
	      if(parent.hasAttribute("href") || 
		   parent.hasAttribute("onclick") || 
               parent.form)
	 	  nodeClk = parent;
         }
         parent = parent.parentNode;
	 }
	} 
      if(nodeClk != null)
      {
        //TBrowseGlobal.tbLogMsg("Found: "+target.nodeName+" "+(target));
        this.lastCrawl = event.target;
        this.lastCrawlResult = nodeClk;
      }  
      if(nodeClk == null)
		{
		 //TBrowseGlobal.tbLogMsg("Stopped: "+target.nodeName+" "+(target));
		 nodeClk = event.originalTarget;
		}
     return nodeClk;
     }    
   },
   /* Ported from the contentAreaClick function and applies to onmouseup.
      Doesn't fire for enter events in textboxes, so we have a keywatcher
   */
   clickEvent: function(event)
   {
     if(event.target instanceof XULElement)
      return;
     //right button has been released
     if(event.target instanceof HTMLImageElement && event.button == 2)
        ThunderBrowseHandler.gestureHold = false; 
     //Checkbox clicking tests
     if(event.target instanceof HTMLInputElement && event.target.type == "checkbox")
      event.target.checked = !event.target.checked;
     //for non-checkbox stuff
     else
     {
	  
      //prevents breaking other extensions. Gesture hold is true when the right button is held down.
      if(this.gestureHold == false)
      {
       TBrowserCore.toggleLaunch(false);
       //TBrowseGlobal.tbLogMsg("Turn off via click event.");
       //TBrowseGlobal.tbLogMsg("Grabbed: "+event.target.nodeName+" "+(event.target));
       var linkSend = (event.target.hasAttribute("onclick")) ? event.target : null;
       handleLinkClick(event,this.getHrefForEvent(event),linkSend);
      } 
     } 
   },
   /* Maintains compatibility with other extensions */
   watchMouseHolds: function(event)
   {
       if(event.target instanceof HTMLImageElement && event.button == 2)
	   ThunderBrowseHandler.gestureHold = true;                                        
   },
   /* Our key watcher. Watches for the enter button, submits the form. */
   keyEvent: function(event)
   {
	  this.keySubmit = false;
     TBrowserCore.toggleLaunch(false);
     var nodeClk = ThunderBrowseHandler.treeCrawler(event);
     if((event.keyCode == KeyEvent.DOM_VK_RETURN || event.keyCode == KeyEvent.DOM_VK_ENTER) 
        && event.shiftKey == false)
     {
      /*
      Problem hits here. If a textbox has a onkeydown that fires a click event, it will double load.
      After several block events, nothing stops this code from running. Somehow it runs at a higher
      process that allows it to load external browsers even though I say no.
      */
      TBrowserCore.toggleLaunch(false);
      if((nodeClk instanceof HTMLInputElement || nodeClk instanceof HTMLButtonElement))
      {
		 if(nodeClk.type != "reset" && nodeClk.type != "radio" && nodeClk.type != "checkbox")
		 {
			 event.preventDefault();
			 this.addOrig = false;
			 this.keySubmit = true;
			 ThunderBrowseHandler.handleFormData(event,this.getHrefForEvent(event),nodeClk);
		 }
		 else if(nodeClk.type == "reset")
			nodeClk.form.reset();
		else if(nodeClk.type == "checkbox")
			nodeClk.checked == !nodeClk.checked;
		 else if(nodeClk.type == "radio")
			this.handleRadioButtons(nodeClk);
      }
      /* Allow users to launch websites using TAB and ENTER */
      else if(nodeClk instanceof HTMLInputElement == false && 
              nodeClk instanceof HTMLButtonElement == false && 
              nodeClk instanceof HTMLTextAreaElement == false)
      {
         event.preventDefault();
         var href = this.getHrefForEvent(event);
         if(href != null || nodeClk.hasAttribute("onclick") == true)
          ThunderBrowseHandler.handleLinks(event,href,nodeClk);
      }
     }
     else
      TBrowserCore.toggleLaunch(true);  
   },
   /* Handles control clicked elements. Requires event and href */
   ctrlClkEvent : function(event, href)
   {
     	if(event.button == 0 && event.ctrlKey == true && 
         thunderbrowseprefs.getBoolPref("loadother") == false && 
         (/(^pop3|^pop|^imap|^mailbox)/i).test(href) == false)
	{
	  var ctrlAssignd2 = thunderbrowseprefs.getIntPref("ctrlclk");
	  /*
	   0 = load normal
	   1 = load tab
    	2 = load in external
    	3 = load in popup
	  */
	  switch(ctrlAssignd2)
	  {
	   case 0: 
	       this.regularLinkLoad(href);
	       break;
	   case 1:
	       ThunderTabBrowser.addTab(href);
	       break;
	   case 2:
	       ThunderBrowseHandler.loadInExternalBrowser(href);
	       break;
	   case 3:
	       TBrowseGlobal.newWindow(href);        
	       break;
           default:
               return false;
	  }	
	}
	else
	return false;   
   },
   /* 
      Handles onClick attributes. Requires all arguments.
      Returns nothing. Decides what code should run.
      A check to see if JS is enabled must be called
      before running this.
   */
   handleOnClick : function(event,href,linkNode)
   {
		var scriptCode, runFurth, javascriptURL = false;
      if((/(^javascript)/i).test(href) == true)
      {
    	 scriptCode = href.replace("javascript:","","i");
		 runFurth = false;  
         javascriptURL = true;
    	}
   	else
   	{
       event.preventDefault();
       scriptCode = linkNode.getAttribute("onclick").replace("javascript:","","i");
	   runFurth = (scriptCode.indexOf("return") != -1) ? false : (linkNode.hasAttribute("href"));
       javascriptURL = false;
      }
      TBrowserCore.toggleLaunch(false);
      //TBrowseGlobal.tbLogMsg("Turn off via onClick");
      try
      {
       checkBrowser();
		 scriptCode = scriptCode.replace("%20"," ", "ig");
        var fakeMouse = document.createEvent("MouseEvents");
        fakeMouse.initMouseEvent("click", false, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
        event.target.dispatchEvent(fakeMouse);
       if(runFurth == true && linkNode && linkNode.hasAttribute("onclick"))
            ThunderBrowse.loadURL(href);
       }
       catch(e)
       {
  	   //TBrowseGlobal.tbLogMsg("onClick code threw exception: "+e);
	   event.preventDefault();
         if(linkNode && javascriptURL == false && runFurth == true)
           ThunderBrowse.loadURL(href);
       }
	if(runFurth == false)
	{
		 //TBrowseGlobal.tbLogMsg("onClick code stopped");
		 event.preventDefault();
	}
   },
   handleOnChange: function(event, href, nodeClk)
   {
    var onChangeCode = nodeClk.parentNode.getAttribute("onchange");
    if((/submit/ig).test(onChangeCode) == true)
    {
     //TBrowseGlobal.tbLogMsg("onChange submitted form.");
     ThunderBrowseHandler.addOrig = false;
     ThunderBrowseHandler.handleFormData(event,href,nodeClk.parentNode);
    }  
    else
    {   
      //TBrowseGlobal.tbLogMsg("trying to run onChange.");
      var onChangeEvent = document.createEvent("HTMLEvents");
      onChangeEvent.initEvent("change", false, true);
      nodeClk.parentNode.dispatchEvent(onChangeEvent);
    }   
   },
   //checks to see if we are on the same domain.
   sameDomain: function(href)
   {
	  try
	  {
        var theHost = RegExp(TBrowseDev.getHost().toString(), "i");
		  return theHost.test(href);
	  }
	  catch(e){return true;}
   },
   handleOnSubmit: function(event, nodeClk)
   {
		TBrowserCore.toggleLaunch(false);
		try
		{
          //TBrowseGlobal.tbLogMsg("onSubmit firing");
          var submitHandler = document.createEvent("HTMLEvents");
          submitHandler.initEvent("submit", false, true);
          nodeClk.form.dispatchEvent(submitHandler);
          return !submitHandler.getPreventDefault();
		}
		catch(e)
		{
		 //TBrowseGlobal.tbLogMsg("onSubmit encountered error: "+e);
		 return true;	
		}
   },
   /* Finds out what we should do with the element that was clicked
      Requires an event and href
   */
   handleLinks : function(event,href,linkNode)
   {
   var target = event.target;
	//0 = left click
	//1 = middle mouse
	if(ThunderBrowseHandler.ctrlClkEvent(event,href) == false)
	{
		//the following code is for middle clicks
		if(event.button == 1 || thunderbrowseprefs.getBoolPref("loadother") == true)
		{
		  if(thunderbrowseprefs.getBoolPref("middletabs") == true && 
			thunderbrowseprefs.getBoolPref("loadother") == false)
		  {
			  ThunderTabBrowser.addTab(href);
		  }
		  if(thunderbrowseprefs.getBoolPref("middletabs") == false || 
			thunderbrowseprefs.getBoolPref("loadother") == true)
		  {
				ThunderBrowseHandler.loadInExternalBrowser(href);
				return false;
		  }
		}
		else if(((/(^pop3|^pop|^imap|^mailbox)/i).test(href) == true))
		{
	          //links in emails. To remain compatible with all modifications, especially download headers
	          //BUG 209
		  if(href.split("#").length == "1")
			gDBView.doCommand(nsMsgViewCommandType.downloadSelectedForOffline);
		  else
			ThunderBrowseHandler.regularLinkLoad(href);     			
		}
		else if(((/(^javascript)/i).test(href) == true || linkNode.hasAttribute("onclick")) && 
            (thunderbrowsejsprefs.getBoolPref("mailnews") == true || TBrowsePermissions.jsAllowed() == true))
			     ThunderBrowseHandler.handleOnClick(event,href,linkNode);
		else
		{
		  this.regularLinkLoad(href);
		}
	}   
   },
   regularLinkLoad: function(href)
   {
	   var isFromMail = TBrowseGlobal.mailProtocols.test(tbrowser.currentURI.scheme);
    	if((thunderbrowseprefs.getBoolPref("lefttab") == true && isFromMail == false && this.sameDomain(href) == true) || (TBrowseTabChooser.gettabType() != 2 && TBrowseTabChooser.isThunderBrowseTab == true && thunderbrowseprefs.getBoolPref("lefttab") == false))
        ThunderBrowse.loadURL(href);
	else if(
        ((ThunderTabBrowser && thunderbrowseprefs.getBoolPref("lefttab") == true) && (isFromMail == true || this.sameDomain(href) == false)) ||
        (ThunderTabBrowser && TBrowseTabChooser.gettabType() == 2 && document.documentElement.getAttribute("windowtype") != "mail:messageWindow" && TBrowseTabChooser.isThunderBrowseTab == false))
	     ThunderTabBrowser.addTab(href);
	else
	  ThunderBrowse.loadURL(href);
   },
	handleRadioButtons : function(nodeClk)
	{
		var listOfInput = nodeClk.form.getElementsByTagName("INPUT");
		for(var i = 0; i < listOfInput.length; i++)
		{
			if(listOfInput[i].type == "radio" && listOfInput[i].name == nodeClk.name)
				listOfInput[i].checked = false;
		}
		nodeClk.checked = true;	
	},
   /* Handles form data submission. Both GET and POST. */
   handleFormData : function(event, href, linkNode)
   {
	checkBrowser();
    if(!linkNode.form)
        return;
        
	var addSubmit = null;
	TBrowserCore.toggleLaunch(false);
	formbaseURL = "";
	this.nodeId = null;
	
	//oddly, some websites put their action type to "null" as a string
	if(linkNode.form.action != null && 
         linkNode.form.action != "null" && 
         linkNode.form.action != "")
	       formbaseURL = linkNode.form.action;
	else
	     formbaseURL = tbrowser.currentURI.spec.split("?")[0];
    
    //See if we should actually continue.
    if(linkNode.form.hasAttribute("onsubmit") == true)
    {
		if(ThunderBrowseHandler.handleOnSubmit(event, linkNode) == false)
            return false;
    }
    
	if(formbaseURL)
	{
	      try{var methd = linkNode.form.method;}
		catch(f){var methd = null;}
		//See if the form method is a GET
		if(!methd || methd == null || methd == "get" || methd == "GET")
		{
		   event.preventDefault();       
		  	var formHref = formbaseURL+"?"; 
		  	var inputtags = linkNode.form.elements;
			var foundSubmit = false;
		  	for (var i = 0; i <= inputtags.length-1; i++) 
		  	{
		  	 if(inputtags[i].hasAttribute("name"))
			 {			 
               if((inputtags[i].type != "submit" && inputtags[i].type != "image" && 
						inputtags[i].type != "reset" && inputtags[i].type != "radio" &&
						inputtags[i].type != "button" && inputtags[i].type != "checkbox") || ((inputtags[i].type == "checkbox" || inputtags[i].type == "button") && inputtags[i].checked == true))
							formHref += inputtags[i].name+"="+encodeURIComponent(inputtags[i].value)+"&";
			 }
			 if(inputtags[i].type == "submit" && foundSubmit == false && this.addOrig == false)
			 {
			   foundSubmit = true;
			   this.nodeId = i;     		  	
			 }
		  	}
		  	if(this.addOrig == false)
	  	   {
	    	    addSubmit = (this.nodeId != null) ? this.nodeId : inputtags.length - 1;
	  	       formHref += inputtags[addSubmit].name+"="+encodeURIComponent(inputtags[addSubmit].value);	  	 
		  	}
		  	//add the submit field if we should.
		  	if(event.originalTarget.name && event.originalTarget.value && this.addOrig == true)
		  		formHref += event.originalTarget.name+"="+event.originalTarget.value;
			
			//Seriously, I have no idea why it doesn't load on keyboard events. This is extremely mind boggling.
			if(this.keySubmit == true)
				setTimeout(ThunderBrowseHandler.regularLinkLoad, 5, formHref);
			ThunderBrowseHandler.regularLinkLoad(formHref);
	  	}	
		//Otherwise it's a POST
		else if(methd == "post" || methd == "POST")
		{ 
			var dataString = "";
			event.preventDefault();
			postbrowserfortab = ThunderTabBrowser.getBrowserForTab(ThunderTabBrowser.selectedTab);
			var inputtags = linkNode.form.elements;
			for (var i = 0; i < inputtags.length; i++) 
			{
				  /*
					Oddly, by using encodeURIComponent, we get a massive fail on some sites.
						It's odd because they accept the encoding of & and + but not spaces
							and other characters.
				  */
				if(inputtags[i].value != null)
					var correctvalue = this.encodeCharacters(inputtags[i].value);  
				if((inputtags[i].type == "radio" || inputtags[i].type == "checkbox") && inputtags[i].checked == true)
					dataString += inputtags[i].name+"="+inputtags[i].value+"&";
				else if(inputtags[i].type != "submit" && 
									  inputtags[i].type != "image" && 
									  inputtags[i].type != "reset" &&
									  inputtags[i].type != "button" &&
									  inputtags[i].type != "radio" && 
				  					  inputtags[i].type != "checkbox")
				  dataString += inputtags[i].name+"="+correctvalue+"&";			 
				  
				if(inputtags[i].type == "submit" && this.addOrig == false)
					this.nodeId = i; 
			 }
			 if(this.addOrig == false)
			 {
				 if(this.nodeId != null)
				  addSubmit = this.nodeId;
				 else 
				  addSubmit = inputtags.length - 1;
				 dataString += inputtags[addSubmit].name+"="+this.encodeCharacters(inputtags[addSubmit].value);	  	 
			 }
			 if(event.originalTarget.name && event.originalTarget.value && this.addOrig == true)
			  dataString += event.originalTarget.name+"="+event.originalTarget.value;
			 var charset = TBrowseGlobal.getCharset(postbrowserfortab);
					 
			 var postData = ThunderBrowseHandler.handlePostData(dataString, charset);
			 var uriRef = TBrowseDev.formatRefUri(formbaseURL);
			 TBHistory.captureRecentUrls(postbrowserfortab);
			 TBHistory.getBackHistory(postbrowserfortab);
			 postbrowserfortab.loadURIWithFlags(formbaseURL, 0, uriRef, charset, postData);
      }
	}   
   },
   /* encodeCharacters and handlePostData both are used in handling POST submissions 
      Encodes special characters for POST requests
      Makes sure the user submits safe data. 
      Returns the encoded string */
   encodeCharacters: function(stringToEncode)
   {
		/* 
		  Instead of using encodeURIComponent, we replace & + = and ; in the strings.
		This works to a degree and solves most of the problems that we originally had.
		However, if the form is powered via AJAX, it does not get cleansed by this.
		*/
	  return stringToEncode.replace(/&/g,"%26").replace(/[+]/g,"%2B").replace(/[=]/g,"%3D").replace(/[;]/g,"%3B");
   },
   /* Takes a string of data (string1=value1&string2=value2)
      and formats it for POST submission. Returns formatted
      data.   */   
   handlePostData : function(stringofData, aCharset)
   {
      var dataString = stringofData;
	var charConverter = Ccl["@mozilla.org/intl/scriptableunicodeconverter"]
                          .createInstance(Cil.nsIScriptableUnicodeConverter);
      
      charConverter.charset = aCharset;
	var stringStream = charConverter.convertToInputStream(dataString);
	var postData = Ccl["@mozilla.org/network/mime-input-stream;1"]
	.createInstance(Cil.nsIMIMEInputStream);
	postData.addHeader("Accept-Language","*");
	postData.addHeader("Accept-Charset","*");
	postData.addHeader("Content-Type", "application/x-www-form-urlencoded");
	postData.addContentLength = true;
	postData.setData(stringStream);
	return postData;   
   },
   /*
     The following functions, skipTBrowser and loadInExternalBrowser handle
     ThunderBrowse's external browser launching.
   */
   crawlSuperTree: function(eventStart)
   {
   var nodeClk, parent, grabbed = false, target = eventStart;
	if(target instanceof HTMLAnchorElement || 
      target instanceof HTMLAreaElement || 
      target instanceof HTMLLinkElement)
	{
	 if (target.hasAttribute("href"))
	 {
	  grabbed = true;
	  nodeClk = target;
	 }
	}
      if(nodeClk == null)
	{    
	 parent = target.parentNode;
	 while (parent && nodeClk == null) 
	 {
	  if ((parent instanceof HTMLAnchorElement || 
             parent instanceof HTMLAreaElement ||
	       parent instanceof HTMLLinkElement) && nodeClk == null)
	   {
	      if (parent.hasAttribute("href"))
		{
		   grabbed = true;
	 	   nodeClk = parent; 
		}
           }
	   parent = parent.parentNode;
	  }
        if(grabbed == false)
	  {
         var children = target.childNodes;
	   for(var i = 0; i < children.length; i++) 
	   {
	    var theChild = children[i];
	    if ((theChild instanceof HTMLAnchorElement || theChild instanceof HTMLAreaElement ||
	 		     theChild instanceof HTMLLinkElement) && nodeClk == null && grabbed == false)
	    {
	      if (theChild.hasAttribute("href"))
	      {
	 	   nodeClk = theChild; 
		   grabbed = true;
	      }
	    }
	   }
   	  }
	 }
        if(grabbed == true)
         return nodeClk.href;
        else
	 return null;
   },
   skipTBrowser: function(outerHref)
   {
   checkBrowser();
   var urltolaunch = null;
	if(outerHref)
		ThunderBrowseHandler.loadInExternalBrowser(outerHref);
	//get the url of the clicked object
	var findUrl = ThunderBrowseHandler.crawlSuperTree(document.popupNode);
	if(tbrowser.currentURI.scheme != "mailbox" && findUrl == null)
          urltolaunch = TBrowseGlobal.getCurrentURL();
	else if(findUrl != null)
	  urltolaunch = findUrl;
	var confirmLoad = null;
	if(thunderbrowseprefs.getBoolPref("askthem") == true && thunderbrowseprefs.getBoolPref("loadother") == false)
		confirmLoad = confirm($("tburlstrings").getString('gotonewbrowse'));
	else
		confirmLoad = true;
	if(confirmLoad == true)
	    ThunderBrowseHandler.loadInExternalBrowser(urltolaunch);
	if(confirmLoad == false)
	    return 0;	 
   },
   /* Launches the url passed to another browser.
      Returns nothing
   */   
   loadInExternalBrowser: function(urlcaptured)
   {
    //Turn on the launcher
    TBrowserCore.toggleLaunch(true);
    try
    {
        //if custom browser path is defined
	if(thunderbrowseprefs.getCharPref("customlauncher") != null && thunderbrowseprefs.getCharPref("customlauncher") != "")
	{
		var browsertolaunch = thunderbrowseprefs.getCharPref("customlauncher").toLowerCase();
		//if it contains %Drive%
		if(browsertolaunch.match("%drive%") != null)
		{
		  var curdrive = Ccl["@mozilla.org/file/directory_service;1"].createInstance(Cil.nsIProperties)
				.get("CurProcD", Cil.nsIFile).path.substring(0, 2);
		  var browsertolaunch = browsertolaunch.replace("%drive%",curdrive);
		}
		var browser = Ccl["@mozilla.org/file/local;1"].createInstance(Cil.nsILocalFile);
		browser.initWithPath(browsertolaunch);
		var bProcess = Ccl["@mozilla.org/process/util;1"].createInstance(Cil.nsIProcess);
		bProcess.init(browser);
		var args = [urlcaptured];
		bProcess.run(false, args, args.length);
	}
	else
	{
	 //launch if not defined.
	 var ioservice = Ccl["@mozilla.org/network/io-service;1"].getService(Cil.nsIIOService);
	 var uriToOpen = ioservice.newURI(urlcaptured, null, null);
	 var extps = Ccl["@mozilla.org/uriloader/external-protocol-service;1"].getService(Cil.nsIExternalProtocolService);
	 extps.loadURI(uriToOpen, null);     
	}
	return true;
    }
    catch(e)
    {
  	//any problems will make it launch anyways
   	var ioservice = Ccl["@mozilla.org/network/io-service;1"].getService(Cil.nsIIOService);
      var uriToOpen = ioservice.newURI(urlcaptured, null, null);
      var extps = Ccl["@mozilla.org/uriloader/external-protocol-service;1"].getService(Cil.nsIExternalProtocolService);
      extps.loadURI(uriToOpen, null);
	return true; 
    }  
  },
  
  getHrefForEvent : function(event)
  {
    //Working around the empty string return. Thanks for the tip Rob!
    var href = hRefForClickEvent(event);
    if(href != null && href.length == 0)
      return null;
    else
      return href;
  }
};
/* The following are either Firefox or Thunderbird functions. They are not wrapped for a reason */
function handleLinkClick(event,href,nodeClk) 
{
     checkBrowser();
     if(nodeClk == null)
      var nodeClk = ThunderBrowseHandler.treeCrawler(event);
     
     if(nodeClk instanceof XULElement)
      return;
      
     TBrowserCore.toggleLaunch((event.button == 2));
     /* Checks for onChange elements */
     if(typeof nodeClk.parentNode != 'undefined' && nodeClk.parentNode != null)
     {
       //Strangest bug.
       if(nodeClk.parentNode.hasAttribute("onchange") == true)
       {
         if(nodeClk.parentNode.getAttribute("onchange").match(/submit/ig) != null)
          event.preventDefault();
     	 ThunderBrowseHandler.handleOnChange(event,href,nodeClk);
       }	        
	  
     }	
    
 	if((event.button == 0 || event.button == 1)  && 
       (nodeClk instanceof HTMLInputElement == false && 
        nodeClk instanceof HTMLButtonElement == false) && 
       (href != null || nodeClk.hasAttribute("onclick") == true))
 	{
 	    event.preventDefault();
       ThunderBrowseHandler.handleLinks(event,href,nodeClk);
   }
 	else if (nodeClk instanceof HTMLInputElement || 
                 nodeClk instanceof HTMLButtonElement || 
                 nodeClk.form)
	{
		if(nodeClk.type == "submit" || 
                 nodeClk.type == "image" || 
                 nodeClk.type == "button")
 	  {
		  //to stop from potentially wasting memory on preparation to submit a form
		  //and then not actually submitting the form we do this
		  event.preventDefault();
		  ThunderBrowseHandler.addOrig = true;
		  ThunderBrowseHandler.handleFormData(event,href,nodeClk);
	  }
	  else if(nodeClk.type == "reset")
			nodeClk.form.reset();
	  else if(nodeClk.type == "checkbox")
			nodeClk.checked == !nodeClk.checked;
	  else if(nodeClk.type == "radio")
			ThunderBrowseHandler.handleRadioButtons(nodeClk);
   }
}
function contentAreaClick(event) 
{
 /* ThunderBrowse enabled browsers inject their own watchers, 
    so we need to stop this one. */
 if(event.button != 2 && event.target instanceof XULElement == false)
  event.preventDefault();
}
//Function that is in Firefox and we're porting what we want to TB
function openUILinkIn(url, where, allowThirdPartyFixup, postData, referrerUrl)
{
 if(!url || !where)
  return;
 if(allowThirdPartyFixup == true)
  TBrowseGlobal.tbLogMsg("ThirdPartyFixup is not supported yet. Sorry.");   
 switch(where) {
   case "current":
   checkBrowser();
   TBHistory.captureRecentUrls(tbrowser);
   TBHistory.getBackHistory(tbrowser);
   ThunderBrowse.launchURL(url, tbrowser, referrerUrl, postData, null);
   break;
   case "tab":
   ThunderTabBrowser.addTab(url);
   break;
   case "tabshifted":
   ThunderTabBrowser.addTab(url,referrerUrl,null,postData,true);
   break;
   case "window":
   TBrowseGlobal.newWindow(url);
   break;
   case "closed":
   TBrowseGlobal.tbLogMsg("lol");
   break;
   case "file":
   saveURL(url, null, null, true, null, referrerUrl); 
   break;
   case "save":
   saveURL(url, null, null, true, null, referrerUrl); 
   break;
  }  
}
function openUILink(url, event)
{
 if(url)
 {
  event.preventDefault();
  if(event.button == 1)
   ThunderTabBrowser.addTab(url);
  if(event.button == 0)
   ThunderBrowseHandler.regularLinkLoad(url);
 }
}
function openLinkExternally(url)
{
  ThunderBrowseHandler.regularLinkLoad(url);
  //TBrowseGlobal.newWindow(url);
}